@@ -13,8 +13,8 @@ install_all_dependencies <- function() {
   # packages and dependencies to install
   pkgs <- c(
     "boot", "car", "data.table", "effsize", "ggplot2",
-    "ggridges", "interactions", "lemon", "lm.beta",
-    "mediation", "moments", "paran", "remotes", "weights",
+    "ggridges", "gridExtra", "interactions", "lemon", "lm.beta",
+    "mediation", "moments", "paran", "psych", "remotes", "weights",
     "WRS2")
   # prep the packages
   kim::prep(pkgs, pkg_names_as_object = TRUE)
