@@ -11,7 +11,7 @@
 #' @param data a data object (a data frame or a data.table)
 #' @param x_var_name name of the variable that will go on the x axis
 #' @param y_var_name name of the variable that will go on the y axis
-#' @param point_label_var_name name of the variable that will be used to
+#' @param dot_label_var_name name of the variable that will be used to
 #' label individual observations
 #' @param weight_var_name name of the variable by which to weight
 #' the individual observations for calculating correlation and plotting
@@ -40,15 +40,16 @@
 #' @param ci_for_line_of_fit if \code{ci_for_line_of_fit = TRUE},
 #' confidence interval for the line of fit will be shaded
 #' @param line_of_fit_color color of the line of fit (default = "blue")
+#' @param dot_color color of the dots (default = "black")
 #' @param x_axis_label alternative label for the x axis
 #' @param y_axis_label alternative label for the y axis
-#' @param point_size size of the dots on the plot (default = 2)
-#' @param point_label_size size for dots' labels on the plot. If no
+#' @param dot_size size of the dots on the plot (default = 2)
+#' @param dot_label_size size for dots' labels on the plot. If no
 #' input is entered for this argument, it will be set as
-#' \code{point_label_size = 5} by default. If the plot is to be
+#' \code{dot_label_size = 5} by default. If the plot is to be
 #' weighted by some variable, this argument will be ignored, and
-#' dot sizes will be determined by the argument \code{point_size_range}
-#' @param point_size_range minimum and maximum size for dots
+#' dot sizes will be determined by the argument \code{dot_size_range}
+#' @param dot_size_range minimum and maximum size for dots
 #' on the plot when they are weighted
 #' @param jitter_x_percent horizontally jitter dots by a percentage of the
 #' range of x values
@@ -56,20 +57,20 @@
 #' range of y values
 #' @param cap_axis_lines logical. Should the axis lines be capped at the
 #' outer tick marks? (default = TRUE)
+#' @param color_dots_by name of the variable that will determine
+#' colors of the dots
 #' @return the output will be a scatter plot, a ggplot object.
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' scatterplot(data = mtcars, x_var_name = "wt", y_var_name = "mpg")
 #' scatterplot(
 #'   data = mtcars, x_var_name = "wt", y_var_name = "mpg",
-#'   point_label_var_name = "hp", weight_var_name = "drat",
-#'   annotate_stats = TRUE
-#' )
+#'   dot_label_var_name = "hp", weight_var_name = "drat",
+#'   annotate_stats = TRUE)
 #' scatterplot(
 #'   data = mtcars, x_var_name = "wt", y_var_name = "mpg",
-#'   point_label_var_name = "hp", weight_var_name = "cyl",
-#'   point_label_size = 7, annotate_stats = TRUE
-#' )
+#'   dot_label_var_name = "hp", weight_var_name = "cyl",
+#'   dot_label_size = 7, annotate_stats = TRUE)
 #' }
 #' @export
 #' @import data.table
@@ -77,7 +78,7 @@ scatterplot <- function(
   data = NULL,
   x_var_name = NULL,
   y_var_name = NULL,
-  point_label_var_name = NULL,
+  dot_label_var_name = NULL,
   weight_var_name = NULL,
   alpha = 1,
   annotate_stats = TRUE,
@@ -88,14 +89,18 @@ scatterplot <- function(
   line_of_fit_type = "lm",
   ci_for_line_of_fit = FALSE,
   line_of_fit_color = "blue",
+  dot_color = "black",
   x_axis_label = NULL,
   y_axis_label = NULL,
-  point_size = 2,
-  point_label_size = NULL,
-  point_size_range = c(3, 12),
+  dot_size = 2,
+  dot_label_size = NULL,
+  dot_size_range = c(3, 12),
   jitter_x_percent = 0,
   jitter_y_percent = 0,
-  cap_axis_lines = FALSE) {
+  cap_axis_lines = FALSE,
+  color_dots_by = NULL) {
+  # bind the vars locally to the function
+  x <- y <- NULL
   # installed packages
   installed_pkgs <- rownames(utils::installed.packages())
   # check if Package 'ggplot2' is installed
@@ -128,77 +133,114 @@ scatterplot <- function(
   }
   # create a temporary dataset
   dt01 <- data.table::data.table(
-    x = data[[x_var_name]], y = data[[y_var_name]])
-  # add the point label or weight column
-  if (!is.null(point_label_var_name)) {
+    x = data[[x_var_name]],
+    y = data[[y_var_name]])
+  # add the column for colors
+  if (!is.null(color_dots_by)) {
     data.table::set(
-      dt01, j = "point_labels", value = data[[point_label_var_name]])
+      dt01, j = "color", value = data[[color_dots_by]])
+  }
+  # add the point label or weight column
+  if (!is.null(dot_label_var_name)) {
+    data.table::set(
+      dt01, j = "dot_labels", value = data[[dot_label_var_name]])
   }
   if (!is.null(weight_var_name)) {
     data.table::set(
       dt01, j = "weight", value = data[[weight_var_name]])
-  } else {
-    # set weight as 1 if no weight_var_name is given
-    data.table::set(dt01, j = "weight", value = 1)
   }
   # remove na values
   num_of_na_rows <- sum(!stats::complete.cases(dt01))
-  dt02 <- stats::na.omit(dt01)
   if (num_of_na_rows > 0) {
+    dt02 <- stats::na.omit(dt01)
     message(paste0(
       num_of_na_rows,
       " rows were removed because of missing values."))
+  } else {
+    dt02 <- dt01
   }
   # ranges for x and y
   x_range <- max(dt02$x) - min(dt02$x)
   y_range <- max(dt02$y) - min(dt02$y)
   # start ggplot
-  g1 <- ggplot2::ggplot(data = dt02, ggplot2::aes(x = dt02$x, y = dt02$y))
+  g1 <- ggplot2::ggplot(data = dt02, ggplot2::aes(
+    x = x, y = y))
+  # add the color
+  if (!is.null(color_dots_by)) {
+    g1 <- g1 + ggplot2::aes(color = dt02$color)
+  }
   # add jitter
-  pj <- ggplot2::position_jitter(
-    width = jitter_x_percent / 100 * x_range,
-    height = jitter_y_percent / 100 * y_range
-  )
+  if (jitter_x_percent > 0 | jitter_y_percent > 0) {
+    pj <- ggplot2::position_jitter(
+      width = jitter_x_percent / 100 * x_range,
+      height = jitter_y_percent / 100 * y_range)
+  }
   # add point labels or dots
-  if (!is.null(point_label_var_name)) {
-    g1 <- g1 + ggplot2::aes(label = dt02$point_labels)
-    if (is.null(point_label_size)) {
-      g1 <- g1 + ggplot2::geom_text(
-        ggplot2::aes(label = dt02$point_labels, fontface = "bold"),
-        position = pj)
+  if (!is.null(dot_label_var_name)) {
+    g1 <- g1 + ggplot2::aes(label = dt02$dot_labels)
+    if (is.null(dot_label_size)) {
+      # add jitter if necessary
+      if (jitter_x_percent > 0 | jitter_y_percent > 0) {
+        g1 <- g1 + ggplot2::geom_text(
+          ggplot2::aes(label = dt02$dot_labels, fontface = "bold"),
+          position = pj)
+      } else {
+        g1 <- g1 + ggplot2::geom_text(
+          ggplot2::aes(label = dt02$dot_labels, fontface = "bold"))
+      }
     } else {
-      g1 <- g1 + ggplot2::geom_text(
-        ggplot2::aes(label = dt02$point_labels, fontface = "bold"),
-        position = pj,
-        size = point_label_size)
+      # add jitter if necessary
+      if (jitter_x_percent > 0 | jitter_y_percent > 0) {
+        g1 <- g1 + ggplot2::geom_text(
+          ggplot2::aes(label = dt02$dot_labels, fontface = "bold"),
+          position = pj,
+          size = dot_label_size)
+      } else {
+        g1 <- g1 + ggplot2::geom_text(
+          ggplot2::aes(label = dt02$dot_labels, fontface = "bold"),
+          size = dot_label_size)
+      }
     }
   } else {
-    g1 <- g1 + ggplot2::geom_point(
-      position = pj, alpha = alpha, size = point_size)
+    # add jitter if necessary
+    if (jitter_x_percent > 0 | jitter_y_percent > 0) {
+      g1 <- g1 + ggplot2::geom_point(
+        alpha = alpha, size = dot_size, position = pj,
+        color = dot_color)
+    } else {
+      g1 <- g1 + ggplot2::geom_point(
+        alpha = alpha, size = dot_size,
+        color = dot_color)
+    }
   }
   # scale points
   if (!is.null(weight_var_name)) {
     g1 <- g1 + ggplot2::aes(size = dt02$weight)
     g1 <- g1 + ggplot2::scale_size(
-      range = point_size_range, guide = FALSE)
+      range = dot_size_range, guide = FALSE)
   }
   # weighted least squares line
   if (line_of_fit_type %in% c("lm", "loess")) {
-    g1 <- g1 + ggplot2::geom_smooth(
-      formula = y ~ x,
-      method = line_of_fit_type,
-      mapping = ggplot2::aes(weight = dt02$weight),
-      color = line_of_fit_color,
-      se = ci_for_line_of_fit
-    )
+    # weighted or not
+    if (!is.null(weight_var_name)) {
+      g1 <- g1 + ggplot2::geom_smooth(
+        formula = y ~ x,
+        method = line_of_fit_type,
+        mapping = ggplot2::aes(weight = dt02$weight),
+        color = line_of_fit_color,
+        se = ci_for_line_of_fit)
+    } else {
+      g1 <- g1 + ggplot2::geom_smooth(
+        formula = y ~ x,
+        method = line_of_fit_type,
+        color = line_of_fit_color,
+        se = ci_for_line_of_fit)
+    }
   }
   # correlation
   cor_test <- stats::cor.test(dt02[["x"]], dt02[["y"]])
   cor_test_df <- cor_test[["parameter"]][["df"]]
-  cor_test_r <- cor_test[["estimate"]]
-  cor_test_p_value <- cor_test[["p.value"]]
-  weighted_r_text <- ""
-  # weighted correlation
+  # weighted or not
   if (!is.null(weight_var_name)) {
     cor_test <- wtd_cor_function(
       x = dt02$x, y = dt02$y,
@@ -206,6 +248,11 @@ scatterplot <- function(
     cor_test_r <- cor_test[1, "correlation"]
     cor_test_p_value <- cor_test[1, "p.value"]
     weighted_r_text <- "weighted"
+  } else {
+    # correlation
+    cor_test_r <- cor_test[["estimate"]]
+    cor_test_p_value <- cor_test[["p.value"]]
+    weighted_r_text <- ""
   }
   # nice p value
   cor_test_p_value_text <- kim::pretty_round_p_value(
@@ -227,15 +274,15 @@ scatterplot <- function(
           t06 = weighted_r_text
         )
       )))
-    g1 <- g1 + ggplot2::geom_text(ggplot2::aes(
+    g1 <- g1 + ggplot2::annotate(
+      geom = "text",
       x = min(dt02$x) + x_range / 2,
-      y = max(dt02$y) + y_range * annotate_y_pos / 100),
+      y = max(dt02$y) + y_range * annotate_y_pos / 100,
       color = annotated_stats_color,
       label = annotation_01, parse = TRUE,
       hjust = 0.5, vjust = 0.5,
       size = annotated_stats_font_size,
-      fontface = annotated_stats_font_face
-    )
+      fontface = annotated_stats_font_face)
   }
   # axis labels
   if (is.null(x_axis_label)) {
